@@ -1,11 +1,11 @@
 import os.path as osp
-import subprocess
-from time import sleep
 
 import numpy as np
 import pytest
 import torch
 
+import torch_geometric.typing
+from torch_geometric import EdgeIndex
 from torch_geometric.data import Data, HeteroData
 from torch_geometric.loader import NeighborLoader
 from torch_geometric.nn import GraphConv, to_hetero
@@ -22,6 +22,7 @@ from torch_geometric.testing import (
     withPackage,
 )
 from torch_geometric.typing import (
+    WITH_EDGE_TIME_NEIGHBOR_SAMPLE,
     WITH_PYG_LIB,
     WITH_TORCH_SPARSE,
     WITH_WEIGHTED_NEIGHBOR_SAMPLE,
@@ -34,6 +35,23 @@ from torch_geometric.utils import (
     to_undirected,
 )
 
+DTYPES = [
+    pytest.param(torch.int64, id='int64'),
+    pytest.param(torch.int32, id='int32'),
+]
+
+SUBGRAPH_TYPES = [
+    pytest.param(SubgraphType.directional, id='directional'),
+    pytest.param(SubgraphType.bidirectional, id='bidirectional'),
+    pytest.param(SubgraphType.induced, id='induced'),
+]
+
+FILTER_PER_WORKERS = [
+    pytest.param(None, id='auto_filter'),
+    pytest.param(True, id='filter_per_worker'),
+    pytest.param(False, id='filter_in_main'),
+]
+
 
 def is_subset(subedge_index, edge_index, src_idx, dst_idx):
     num_nodes = int(edge_index.max()) + 1
@@ -45,15 +63,21 @@ def is_subset(subedge_index, edge_index, src_idx, dst_idx):
 
 @withCUDA
 @onlyNeighborSampler
-@pytest.mark.parametrize('subgraph_type', list(SubgraphType))
-@pytest.mark.parametrize('dtype', [torch.int64, torch.int32])
-@pytest.mark.parametrize('filter_per_worker', [None, True, False])
-def test_homo_neighbor_loader_basic(device, subgraph_type, dtype,
-                                    filter_per_worker):
+@pytest.mark.parametrize('dtype', DTYPES)
+@pytest.mark.parametrize('subgraph_type', SUBGRAPH_TYPES)
+@pytest.mark.parametrize('filter_per_worker', FILTER_PER_WORKERS)
+def test_homo_neighbor_loader_basic(
+    device,
+    subgraph_type,
+    dtype,
+    filter_per_worker,
+):
+    if dtype != torch.int64 and not torch_geometric.typing.WITH_PT20:
+        return
+    induced = SubgraphType.induced
     if subgraph_type == SubgraphType.induced and not WITH_TORCH_SPARSE:
         return
-    if (dtype != torch.int64
-            and (not WITH_PYG_LIB or subgraph_type == SubgraphType.induced)):
+    if dtype != torch.int64 and (not WITH_PYG_LIB or subgraph_type == induced):
         return
 
     torch.manual_seed(12345)
@@ -86,6 +110,12 @@ def test_homo_neighbor_loader_basic(device, subgraph_type, dtype,
         assert batch.n_id.size() == (batch.num_nodes, )
         assert batch.input_id.numel() == batch.batch_size == 20
         assert batch.x.min() >= 0 and batch.x.max() < 100
+        # TODO Re-enable once `EdgeIndex` is stable.
+        assert not isinstance(batch.edge_index, EdgeIndex)
+        # batch.edge_index.validate()
+        # size = (batch.num_nodes, batch.num_nodes)
+        # assert batch.edge_index.sparse_size() == size
+        # assert batch.edge_index.sort_order == 'col'
         assert batch.edge_index.device == device
         assert batch.edge_index.min() >= 0
         assert batch.edge_index.max() < batch.num_nodes
@@ -113,13 +143,15 @@ def test_homo_neighbor_loader_basic(device, subgraph_type, dtype,
 
 
 @onlyNeighborSampler
-@pytest.mark.parametrize('subgraph_type', list(SubgraphType))
-@pytest.mark.parametrize('dtype', [torch.int64, torch.int32])
+@pytest.mark.parametrize('dtype', DTYPES)
+@pytest.mark.parametrize('subgraph_type', SUBGRAPH_TYPES)
 def test_hetero_neighbor_loader_basic(subgraph_type, dtype):
+    if dtype != torch.int64 and not torch_geometric.typing.WITH_PT20:
+        return
+    induced = SubgraphType.induced
     if subgraph_type == SubgraphType.induced and not WITH_TORCH_SPARSE:
         return
-    if (dtype != torch.int64
-            and (not WITH_PYG_LIB or subgraph_type == SubgraphType.induced)):
+    if dtype != torch.int64 and (not WITH_PYG_LIB or subgraph_type == induced):
         return
 
     torch.manual_seed(12345)
@@ -191,6 +223,15 @@ def test_hetero_neighbor_loader_basic(subgraph_type, dtype):
                                          ('paper', 'to', 'author'),
                                          ('author', 'to', 'paper')}
 
+        for edge_type, edge_index in batch.edge_index_dict.items():
+            src, _, dst = edge_type
+            # TODO Re-enable once `EdgeIndex` is stable.
+            assert not isinstance(edge_index, EdgeIndex)
+            # edge_index.validate()
+            # size = (batch[src].num_nodes, batch[dst].num_nodes)
+            # assert edge_index.sparse_size() == size
+            # assert edge_index.sort_order == 'col'
+
         row, col = batch['paper', 'paper'].edge_index
         assert row.min() >= 0 and row.max() < batch['paper'].num_nodes
         assert col.min() >= 0 and col.max() < batch['paper'].num_nodes
@@ -231,9 +272,11 @@ def test_hetero_neighbor_loader_basic(subgraph_type, dtype):
             assert 'e_id' not in batch['paper', 'author']
             assert 'edge_attr' not in batch['paper', 'author']
 
+            edge_index1 = batch['paper', 'author'].edge_index
+            edge_index2 = batch['author', 'paper'].edge_index
             assert torch.equal(
-                batch['paper', 'author'].edge_index,
-                sort_edge_index(batch['author', 'paper'].edge_index.flip([0])),
+                edge_index1,
+                sort_edge_index(edge_index2.flip([0]), sort_by_row=False),
             )
 
         row, col = batch['author', 'paper'].edge_index
@@ -255,9 +298,11 @@ def test_hetero_neighbor_loader_basic(subgraph_type, dtype):
             assert 'e_id' not in batch['author', 'paper']
             assert 'edge_attr' not in batch['author', 'paper']
 
+            edge_index1 = batch['author', 'paper'].edge_index
+            edge_index2 = batch['paper', 'author'].edge_index
             assert torch.equal(
-                batch['author', 'paper'].edge_index,
-                sort_edge_index(batch['paper', 'author'].edge_index.flip([0])),
+                edge_index1,
+                sort_edge_index(edge_index2.flip([0]), sort_by_row=False),
             )
 
         # Test for isolated nodes (there shouldn't exist any):
@@ -266,11 +311,11 @@ def test_hetero_neighbor_loader_basic(subgraph_type, dtype):
 
 @onlyOnline
 @onlyNeighborSampler
-@pytest.mark.parametrize('subgraph_type', list(SubgraphType))
-def test_homo_neighbor_loader_on_cora(get_dataset, subgraph_type):
+@pytest.mark.parametrize('subgraph_type', SUBGRAPH_TYPES)
+def test_homo_neighbor_loader_on_karate(get_dataset, subgraph_type):
     if subgraph_type == SubgraphType.induced and not WITH_TORCH_SPARSE:
         return
-    dataset = get_dataset(name='Cora')
+    dataset = get_dataset(name='karate')
     data = dataset[0]
 
     mask = data.edge_index[0] < data.edge_index[1]
@@ -312,16 +357,16 @@ def test_homo_neighbor_loader_on_cora(get_dataset, subgraph_type):
 
 @onlyOnline
 @onlyNeighborSampler
-@pytest.mark.parametrize('subgraph_type', list(SubgraphType))
-def test_hetero_neighbor_loader_on_cora(get_dataset, subgraph_type):
+@pytest.mark.parametrize('subgraph_type', SUBGRAPH_TYPES)
+def test_hetero_neighbor_loader_on_karate(get_dataset, subgraph_type):
     if subgraph_type == SubgraphType.induced and not WITH_TORCH_SPARSE:
         return
-    dataset = get_dataset(name='Cora')
+    dataset = get_dataset(name='karate')
     data = dataset[0]
 
     hetero_data = HeteroData()
-    hetero_data['paper'].x = data.x
-    hetero_data['paper', 'paper'].edge_index = data.edge_index
+    hetero_data['v'].x = data.x
+    hetero_data['v', 'v'].edge_index = data.edge_index
 
     split_idx = torch.arange(5, 8)
 
@@ -329,13 +374,13 @@ def test_hetero_neighbor_loader_on_cora(get_dataset, subgraph_type):
         hetero_data,
         num_neighbors=[-1, -1],
         batch_size=split_idx.numel(),
-        input_nodes=('paper', split_idx),
+        input_nodes=('v', split_idx),
         subgraph_type=subgraph_type,
     )
     assert len(loader) == 1
 
     hetero_batch = next(iter(loader))
-    batch_size = hetero_batch['paper'].batch_size
+    batch_size = hetero_batch['v'].batch_size
 
     class GNN(torch.nn.Module):
         def __init__(self, in_channels, hidden_channels, out_channels):
@@ -353,27 +398,26 @@ def test_hetero_neighbor_loader_on_cora(get_dataset, subgraph_type):
 
     out1 = model(data.x, data.edge_index)[split_idx]
     out2 = hetero_model(hetero_batch.x_dict,
-                        hetero_batch.edge_index_dict)['paper'][:batch_size]
+                        hetero_batch.edge_index_dict)['v'][:batch_size]
     assert torch.allclose(out1, out2, atol=1e-6)
 
 
 @onlyOnline
 @withPackage('pyg_lib')
-def test_temporal_hetero_neighbor_loader_on_cora(get_dataset):
-    dataset = get_dataset(name='Cora')
+def test_temporal_hetero_neighbor_loader_on_karate(get_dataset):
+    dataset = get_dataset(name='karate')
     data = dataset[0]
 
     hetero_data = HeteroData()
-    hetero_data['paper'].x = data.x
-    hetero_data['paper'].time = torch.arange(data.num_nodes, 0, -1)
-    hetero_data['paper', 'paper'].edge_index = data.edge_index
+    hetero_data['v'].x = data.x
+    hetero_data['v'].time = torch.arange(data.num_nodes, 0, -1)
+    hetero_data['v', 'v'].edge_index = data.edge_index
 
     loader = NeighborLoader(hetero_data, num_neighbors=[-1, -1],
-                            input_nodes='paper', time_attr='time',
-                            batch_size=1)
+                            input_nodes='v', time_attr='time', batch_size=1)
 
     for batch in loader:
-        mask = batch['paper'].time[0] >= batch['paper'].time[1:]
+        mask = batch['v'].time[0] >= batch['v'].time[1:]
         assert torch.all(mask)
 
 
@@ -382,6 +426,51 @@ def test_custom_neighbor_loader():
     # Initialize feature store, graph store, and reference:
     feature_store = MyFeatureStore()
     graph_store = MyGraphStore()
+
+    # Set up node features:
+    x = torch.arange(100, 300)
+    feature_store.put_tensor(x, group_name=None, attr_name='x', index=None)
+
+    y = torch.arange(100, 300)
+    feature_store.put_tensor(y, group_name=None, attr_name='y', index=None)
+
+    # COO:
+    edge_index = get_random_edge_index(100, 100, 500, coalesce=True)
+    edge_index = edge_index[:, torch.randperm(edge_index.size(1))]
+    coo = (edge_index[0], edge_index[1])
+    graph_store.put_edge_index(edge_index=coo, edge_type=None, layout='coo',
+                               size=(100, 100))
+
+    data = Data(x=x, edge_index=edge_index, y=y, num_nodes=200)
+
+    # Construct neighbor loaders:
+    loader1 = NeighborLoader(data, batch_size=20,
+                             input_nodes=torch.arange(100),
+                             num_neighbors=[-1] * 2)
+
+    loader2 = NeighborLoader((feature_store, graph_store), batch_size=20,
+                             input_nodes=torch.arange(100),
+                             num_neighbors=[-1] * 2)
+
+    assert str(loader1) == str(loader2)
+    assert len(loader1) == len(loader2)
+
+    for batch1, batch2 in zip(loader1, loader2):
+        assert len(batch1) == len(batch2)
+        assert batch1.num_nodes == batch2.num_nodes
+        assert batch1.num_edges == batch2.num_edges
+        assert batch1.batch_size == batch2.batch_size
+
+        # Mapped indices of neighbors may be differently sorted ...
+        assert torch.allclose(batch1.x.sort()[0], batch2.x.sort()[0])
+        assert torch.allclose(batch1.y.sort()[0], batch2.y.sort()[0])
+
+
+@onlyNeighborSampler
+def test_custom_hetero_neighbor_loader():
+    # Initialize feature store, graph store, and reference:
+    feature_store = MyFeatureStore()
+    graph_store = MyGraphStore()
     data = HeteroData()
 
     # Set up node features:
@@ -460,9 +549,8 @@ def test_custom_neighbor_loader():
 
 @onlyOnline
 @withPackage('pyg_lib')
-def test_temporal_custom_neighbor_loader_on_cora(get_dataset):
-    # Initialize dataset (once):
-    dataset = get_dataset(name='Cora')
+def test_temporal_custom_neighbor_loader_on_karate(get_dataset):
+    dataset = get_dataset(name='karate')
     data = dataset[0]
     data.time = torch.arange(data.num_nodes, 0, -1)
 
@@ -473,19 +561,19 @@ def test_temporal_custom_neighbor_loader_on_cora(get_dataset):
 
     feature_store.put_tensor(
         data.x,
-        group_name='paper',
+        group_name='v',
         attr_name='x',
         index=None,
     )
-    hetero_data['paper'].x = data.x
+    hetero_data['v'].x = data.x
 
     feature_store.put_tensor(
         data.time,
-        group_name='paper',
+        group_name='v',
         attr_name='time',
         index=None,
     )
-    hetero_data['paper'].time = data.time
+    hetero_data['v'].time = data.time
 
     # Sort according to time in local neighborhoods:
     row, col = data.edge_index
@@ -494,17 +582,17 @@ def test_temporal_custom_neighbor_loader_on_cora(get_dataset):
 
     graph_store.put_edge_index(
         edge_index,
-        edge_type=('paper', 'to', 'paper'),
+        edge_type=('v', 'to', 'v'),
         layout='coo',
         is_sorted=True,
         size=(data.num_nodes, data.num_nodes),
     )
-    hetero_data['paper', 'to', 'paper'].edge_index = data.edge_index
+    hetero_data['v', 'to', 'v'].edge_index = data.edge_index
 
     loader1 = NeighborLoader(
         hetero_data,
         num_neighbors=[-1, -1],
-        input_nodes='paper',
+        input_nodes='v',
         time_attr='time',
         batch_size=128,
     )
@@ -512,13 +600,13 @@ def test_temporal_custom_neighbor_loader_on_cora(get_dataset):
     loader2 = NeighborLoader(
         (feature_store, graph_store),
         num_neighbors=[-1, -1],
-        input_nodes='paper',
+        input_nodes='v',
         time_attr='time',
         batch_size=128,
     )
 
     for batch1, batch2 in zip(loader1, loader2):
-        assert torch.equal(batch1['paper'].time, batch2['paper'].time)
+        assert torch.equal(batch1['v'].time, batch2['v'].time)
 
 
 @withPackage('pyg_lib', 'torch_sparse')
@@ -530,7 +618,7 @@ def test_pyg_lib_and_torch_sparse_homo_equality():
     seed = torch.arange(10)
 
     sample = torch.ops.pyg.neighbor_sample
-    out1 = sample(colptr, row, seed, [-1, -1], None, None, None, True)
+    out1 = sample(colptr, row, seed, [-1, -1], None, None, None, None, True)
     sample = torch.ops.torch_sparse.neighbor_sample
     out2 = sample(colptr, row, seed, [-1, -1], False, True)
 
@@ -570,8 +658,8 @@ def test_pyg_lib_and_torch_sparse_hetero_equality():
 
     sample = torch.ops.pyg.hetero_neighbor_sample
     out1 = sample(node_types, edge_types, colptr_dict, row_dict, seed_dict,
-                  num_neighbors_dict, None, None, None, True, False, True,
-                  False, "uniform", True)
+                  num_neighbors_dict, None, None, None, None, True, False,
+                  True, False, "uniform", True)
     sample = torch.ops.torch_sparse.hetero_neighbor_sample
     out2 = sample(node_types, edge_types, colptr_dict, row_dict, seed_dict,
                   num_neighbors_dict, 2, False, True)
@@ -614,31 +702,6 @@ def test_memmap_neighbor_loader(tmp_path):
     assert batch.x.size() == (batch.num_nodes, 32)
 
 
-@onlyLinux
-@onlyNeighborSampler
-@pytest.mark.parametrize('loader_cores', [None, [1]])
-def test_cpu_affinity_neighbor_loader(loader_cores):
-    data = Data(x=torch.randn(1, 1))
-    loader = NeighborLoader(data, num_neighbors=[-1], batch_size=1,
-                            num_workers=1)
-
-    out = []
-    with loader.enable_cpu_affinity(loader_cores):
-        iterator = loader._get_iterator()
-        workers = iterator._workers
-        for worker in workers:
-            sleep(1)  # Gives time for worker to initialize.
-            process = subprocess.Popen(
-                ['taskset', '-c', '-p', f'{worker.pid}'],
-                stdout=subprocess.PIPE)
-            stdout = process.communicate()[0].decode('utf-8')
-            out.append(int(stdout.split(':')[1].strip()))
-        if not loader_cores:
-            assert out == [0]
-        else:
-            assert out == loader_cores
-
-
 @withPackage('pyg_lib')
 def test_homo_neighbor_loader_sampled_info():
     edge_index = torch.tensor([
@@ -786,6 +849,64 @@ def test_weighted_hetero_neighbor_loader():
     assert global_edge_index.tolist() == [[3, 4], [2, 3]]
 
 
+@pytest.mark.skipif(
+    not WITH_EDGE_TIME_NEIGHBOR_SAMPLE,
+    reason="'pyg-lib' does not support weighted neighbor sampling",
+)
+def test_edge_level_temporal_homo_neighbor_loader():
+    edge_index = torch.tensor([
+        [0, 1, 1, 2, 2, 3, 3, 4],
+        [1, 0, 2, 1, 3, 2, 4, 3],
+    ])
+    edge_time = torch.arange(edge_index.size(1))
+
+    data = Data(edge_index=edge_index, edge_time=edge_time, num_nodes=5)
+
+    loader = NeighborLoader(
+        data,
+        num_neighbors=[-1, -1],
+        input_time=torch.tensor([4, 4, 4, 4, 4]),
+        time_attr='edge_time',
+        batch_size=1,
+    )
+
+    for batch in loader:
+        assert batch.edge_time.numel() == batch.num_edges
+        if batch.edge_time.numel() > 0:
+            assert batch.edge_time.max() <= 4
+
+
+@pytest.mark.skipif(
+    not WITH_EDGE_TIME_NEIGHBOR_SAMPLE,
+    reason="'pyg-lib' does not support weighted neighbor sampling",
+)
+def test_edge_level_temporal_hetero_neighbor_loader():
+    edge_index = torch.tensor([
+        [0, 1, 1, 2, 2, 3, 3, 4],
+        [1, 0, 2, 1, 3, 2, 4, 3],
+    ])
+    edge_time = torch.arange(edge_index.size(1))
+
+    data = HeteroData()
+    data['A'].num_nodes = 5
+    data['A', 'A'].edge_index = edge_index
+    data['A', 'A'].edge_time = edge_time
+
+    loader = NeighborLoader(
+        data,
+        num_neighbors=[-1, -1],
+        input_nodes='A',
+        input_time=torch.tensor([4, 4, 4, 4, 4]),
+        time_attr='edge_time',
+        batch_size=1,
+    )
+
+    for batch in loader:
+        assert batch['A', 'A'].edge_time.numel() == batch['A', 'A'].num_edges
+        if batch['A', 'A'].edge_time.numel() > 0:
+            assert batch['A', 'A'].edge_time.max() <= 4
+
+
 @withCUDA
 @onlyNeighborSampler
 @withPackage('torch_frame')
@@ -814,3 +935,61 @@ def test_neighbor_loader_with_tensor_frame(device):
         assert batch.global_tf.device == device
         assert batch.global_tf.num_rows == 1
         assert batch.global_tf == data.global_tf
+
+
+@onlyNeighborSampler
+def test_neighbor_loader_input_id():
+    data = HeteroData()
+    data['a'].num_nodes = 10
+    data['b'].num_nodes = 12
+
+    row = torch.randint(0, data['a'].num_nodes, (40, ))
+    col = torch.randint(0, data['b'].num_nodes, (40, ))
+    data['a', 'b'].edge_index = torch.stack([row, col], dim=0)
+    data['b', 'a'].edge_index = torch.stack([col, row], dim=0)
+
+    mask = torch.ones(data['a'].num_nodes, dtype=torch.bool)
+    mask[0] = False
+
+    loader = NeighborLoader(
+        data,
+        input_nodes=('a', mask),
+        batch_size=2,
+        num_neighbors=[2, 2],
+    )
+    for i, batch in enumerate(loader):
+        if i < 4:
+            expected = [(2 * i) + 1, (2 * i) + 2]
+        else:
+            expected = [(2 * i) + 1]
+
+        assert batch['a'].input_id.tolist() == expected
+
+
+@withPackage('pyg_lib')
+def test_temporal_neighbor_loader_single_link():
+    data = HeteroData()
+    data['a'].x = torch.arange(10)
+    data['b'].x = torch.arange(10)
+    data['c'].x = torch.arange(10)
+
+    data['b'].time = torch.arange(0, 10)
+    data['c'].time = torch.arange(1, 11)
+
+    data['a', 'b'].edge_index = torch.arange(10).view(1, -1).repeat(2, 1)
+    data['b', 'a'].edge_index = torch.arange(10).view(1, -1).repeat(2, 1)
+    data['a', 'c'].edge_index = torch.arange(10).view(1, -1).repeat(2, 1)
+    data['c', 'a'].edge_index = torch.arange(10).view(1, -1).repeat(2, 1)
+
+    loader = NeighborLoader(
+        data,
+        num_neighbors=[-1],
+        input_nodes='a',
+        time_attr='time',
+        input_time=torch.arange(0, 10),
+        batch_size=10,
+    )
+    batch = next(iter(loader))
+    assert batch['a'].num_nodes == 10
+    assert batch['b'].num_nodes == 10
+    assert batch['c'].num_nodes == 0
