@@ -22,5 +22,5 @@ def test_degree_scaler_aggregation(train_norm):
     assert out.size() == (3, 240)
     assert torch.allclose(torch.jit.script(aggr)(x, index), out)
 
-    with pytest.raises(NotImplementedError):
+    with pytest.raises(NotImplementedError, match="requires 'index'"):
         aggr(x, ptr=ptr)
