@@ -1,3 +1,5 @@
+import math
+
 import torch
 
 from torch_geometric.nn import DMoNPooling
@@ -16,6 +18,6 @@ def test_dmon_pooling():
     assert s.size() == (2, 20, 10)
     assert x.size() == (2, 10, 16)
     assert adj.size() == (2, 10, 10)
-    assert -1 <= spectral_loss <= 0
-    assert 0 <= ortho_loss <= 2
-    assert -1 <= cluster_loss <= 0
+    assert -1 <= spectral_loss <= 0.5
+    assert 0 <= ortho_loss <= math.sqrt(2)
+    assert 0 <= cluster_loss <= math.sqrt(num_clusters) - 1
