@@ -1,11 +1,10 @@
 import torch
 
 from torch_geometric.nn import GraphUNet
-from torch_geometric.testing import is_full_test, onlyLinux, withPackage
+from torch_geometric.testing import is_full_test, onlyLinux
 
 
 @onlyLinux  # TODO  (matthias) Investigate CSR @ CSR support on Windows.
-@withPackage('torch>=1.12.0')
 def test_graph_unet():
     model = GraphUNet(16, 32, 8, depth=3)
     out = 'GraphUNet(16, 32, 8, depth=3, pool_ratios=[0.5, 0.5, 0.5])'
