@@ -3,13 +3,11 @@ import random
 import torch
 from torch import Tensor
 
-import torch_geometric
 from torch_geometric.testing import (
-    disableExtensions,
     get_random_edge_index,
     onlyFullTest,
     onlyLinux,
-    withCUDA,
+    withDevice,
     withPackage,
 )
 from torch_geometric.utils import scatter
@@ -27,14 +25,13 @@ class MySAGEConv(torch.nn.Module):
         return self.lin_src(out) + self.lin_dst(x)
 
 
-@withCUDA
+@withDevice
 @onlyLinux
 @onlyFullTest
-@disableExtensions
 @withPackage('torch>2.0.0')
 def test_dynamic_torch_compile(device):
     conv = MySAGEConv(64, 64).to(device)
-    conv = torch_geometric.compile(conv, dynamic=True)
+    conv = torch.compile(conv, dynamic=True)
 
     optimizer = torch.optim.Adam(conv.parameters(), lr=0.01)
 
