@@ -1,12 +1,12 @@
 import torch
 
+import torch_geometric.typing
 from torch_geometric.nn import GraphSAGE
 from torch_geometric.profile.profiler import Profiler
-from torch_geometric.testing import withCUDA, withPackage
+from torch_geometric.testing import withDevice
 
 
-@withCUDA
-@withPackage('torch>=1.13.0')  # TODO Investigate test errors
+@withDevice
 def test_profiler(capfd, get_dataset, device):
     x = torch.randn(10, 16, device=device)
     edge_index = torch.tensor([
@@ -20,7 +20,8 @@ def test_profiler(capfd, get_dataset, device):
         model(x, edge_index)
 
     _, err = capfd.readouterr()
-    assert 'Completed Stage' in err
+    if not torch_geometric.typing.WITH_PT24:
+        assert 'Completed Stage' in err
 
     _, heading_list, raw_results, layer_names, layer_stats = prof.get_trace()
     assert 'Self CPU total' in heading_list
