@@ -1,9 +1,11 @@
 import torch
 
 from torch_geometric.data import Data
+from torch_geometric.testing import withPackage
 from torch_geometric.transforms import LaplacianLambdaMax
 
 
+@withPackage('scipy')
 def test_laplacian_lambda_max():
     out = str(LaplacianLambdaMax())
     assert out == 'LaplacianLambdaMax(normalization=None)'
