@@ -1,11 +1,11 @@
 import torch
 
 from torch_geometric.data import Data
-from torch_geometric.testing import withCUDA
+from torch_geometric.testing import withDevice
 from torch_geometric.transforms import ToDevice
 
 
-@withCUDA
+@withDevice
 def test_to_device(device):
     x = torch.randn(3, 4)
     edge_index = torch.tensor([[0, 1, 1, 2], [1, 0, 2, 1]])
