@@ -1,15 +1,14 @@
-from collections.abc import Mapping
-from typing import Iterable
+from typing import Any, Iterator, List, Mapping, Tuple
 
 
 class MappingView:
-    def __init__(self, mapping: Mapping, *args: str):
+    def __init__(self, mapping: Mapping[str, Any], *args: str):
         self._mapping = mapping
         self._args = args
 
-    def _keys(self) -> Iterable:
+    def _keys(self) -> List[str]:
         if len(self._args) == 0:
-            return self._mapping.keys()
+            return list(self._mapping.keys())
         else:
             return [arg for arg in self._args if arg in self._mapping]
 
@@ -20,21 +19,21 @@ class MappingView:
         mapping = {key: self._mapping[key] for key in self._keys()}
         return f'{self.__class__.__name__}({mapping})'
 
-    __class_getitem__ = classmethod(type([]))
+    __class_getitem__ = classmethod(type([]))  # type: ignore
 
 
 class KeysView(MappingView):
-    def __iter__(self) -> Iterable:
+    def __iter__(self) -> Iterator[str]:
         yield from self._keys()
 
 
 class ValuesView(MappingView):
-    def __iter__(self) -> Iterable:
+    def __iter__(self) -> Iterator[Any]:
         for key in self._keys():
             yield self._mapping[key]
 
 
 class ItemsView(MappingView):
-    def __iter__(self):
+    def __iter__(self) -> Iterator[Tuple[str, Any]]:
         for key in self._keys():
             yield (key, self._mapping[key])
