@@ -6,8 +6,7 @@ from torch_geometric.graphgym.config import cfg
 
 
 def compute_loss(pred, true):
-    """
-    Compute loss and prediction score
+    """Compute loss and prediction score.
 
     Args:
         pred (torch.tensor): Unnormalized prediction
