@@ -25,8 +25,8 @@ from torch_geometric.nn.resolver import (
     activation_resolver,
     normalization_resolver,
 )
-from torch_geometric.typing import Adj, OptTensor, SparseTensor
-from torch_geometric.utils.trim_to_layer import TrimToLayer
+from torch_geometric.typing import Adj, OptTensor
+from torch_geometric.utils._trim_to_layer import TrimToLayer
 
 
 class BasicGNN(torch.nn.Module):
@@ -175,46 +175,19 @@ class BasicGNN(torch.nn.Module):
         if hasattr(self, 'lin'):
             self.lin.reset_parameters()
 
-    @torch.jit._overload_method
-    def forward(  # noqa
-        x,
-        edge_index,
-        edge_weight=None,
-        edge_attr=None,
-        batch=None,
-        batch_size=None,
-        num_sampled_nodes_per_hop=None,
-        num_sampled_edges_per_hop=None,
-    ):
-        # type: (Tensor, Tensor, OptTensor, OptTensor, OptTensor, Optional[int], Optional[List[int]], Optional[List[int]]) -> Tensor  # noqa
-        pass
-
-    @torch.jit._overload_method
-    def forward(  # noqa
-        x,
-        edge_index,
-        edge_weight=None,
-        edge_attr=None,
-        batch=None,
-        batch_size=None,
-        num_sampled_nodes_per_hop=None,
-        num_sampled_edges_per_hop=None,
-    ):
-        # type: (Tensor, SparseTensor, OptTensor, OptTensor, OptTensor, Optional[int], Optional[List[int]], Optional[List[int]]) -> Tensor  # noqa
-        pass
-
-    def forward(  # noqa
+    def forward(
         self,
         x: Tensor,
-        edge_index: Tensor,  # TODO Support `SparseTensor` in type hint.
+        edge_index: Adj,
         edge_weight: OptTensor = None,
         edge_attr: OptTensor = None,
         batch: OptTensor = None,
         batch_size: Optional[int] = None,
         num_sampled_nodes_per_hop: Optional[List[int]] = None,
         num_sampled_edges_per_hop: Optional[List[int]] = None,
-    ):
-        r"""
+    ) -> Tensor:
+        r"""Forward pass.
+
         Args:
             x (torch.Tensor): The input node features.
             edge_index (torch.Tensor or SparseTensor): The edge indices.
@@ -254,8 +227,8 @@ class BasicGNN(torch.nn.Module):
         xs: List[Tensor] = []
         assert len(self.convs) == len(self.norms)
         for i, (conv, norm) in enumerate(zip(self.convs, self.norms)):
-            if (num_sampled_nodes_per_hop is not None
-                    and not torch.jit.is_scripting()):
+            if (not torch.jit.is_scripting()
+                    and num_sampled_nodes_per_hop is not None):
                 x, edge_index, value = self._trim(
                     i,
                     num_sampled_nodes_per_hop,
@@ -408,84 +381,6 @@ class BasicGNN(torch.nn.Module):
 
         return x_all
 
-    def jittable(self, use_sparse_tensor: bool = False) -> 'BasicGNN':
-        r"""Produces a new jittable instance module that can be used in
-        combination with :meth:`torch.jit.script`."""
-        class EdgeIndexJittable(torch.nn.Module):
-            def __init__(self, child: BasicGNN):
-                super().__init__()
-                self.child = child
-
-            def reset_parameters(self):
-                self.child.reset_parameters()
-
-            def forward(
-                self,
-                x: Tensor,
-                edge_index: Tensor,
-                edge_weight: OptTensor = None,
-                edge_attr: OptTensor = None,
-                batch: OptTensor = None,
-                batch_size: Optional[int] = None,
-                num_sampled_nodes_per_hop: Optional[List[int]] = None,
-                num_sampled_edges_per_hop: Optional[List[int]] = None,
-            ) -> Tensor:
-                return self.child(
-                    x,
-                    edge_index,
-                    edge_weight,
-                    edge_attr,
-                    batch,
-                    batch_size,
-                    num_sampled_nodes_per_hop,
-                    num_sampled_edges_per_hop,
-                )
-
-            def __repr__(self) -> str:
-                return str(self.child)
-
-        class SparseTensorJittable(torch.nn.Module):
-            def __init__(self, child: BasicGNN):
-                super().__init__()
-                self.child = child
-
-            def reset_parameters(self):
-                self.child.reset_parameters()
-
-            def forward(
-                self,
-                x: Tensor,
-                edge_index: SparseTensor,
-                edge_weight: OptTensor = None,
-                edge_attr: OptTensor = None,
-                batch: OptTensor = None,
-                batch_size: Optional[int] = None,
-                num_sampled_nodes_per_hop: Optional[List[int]] = None,
-                num_sampled_edges_per_hop: Optional[List[int]] = None,
-            ) -> Tensor:
-                return self.child(
-                    x,
-                    edge_index,
-                    edge_weight,
-                    edge_attr,
-                    batch,
-                    batch_size,
-                    num_sampled_nodes_per_hop,
-                    num_sampled_edges_per_hop,
-                )
-
-            def __repr__(self) -> str:
-                return str(self.child)
-
-        out = copy.deepcopy(self)
-        convs = [conv.jittable() for conv in out.convs]
-        out.convs = torch.nn.ModuleList(convs)
-        out._trim = None  # TODO Trimming is currently not support in JIT mode.
-
-        if use_sparse_tensor:
-            return SparseTensorJittable(out)
-        return EdgeIndexJittable(out)
-
     def __repr__(self) -> str:
         return (f'{self.__class__.__name__}({self.in_channels}, '
                 f'{self.out_channels}, num_layers={self.num_layers})')
