@@ -2,7 +2,7 @@ import copy
 import math
 import sys
 import warnings
-from typing import Callable, Dict, List, Optional, Tuple, Union
+from typing import Callable, Dict, List, Literal, Optional, Tuple, Union
 
 import torch
 from torch import Tensor
@@ -33,7 +33,8 @@ NumNeighborsType = Union[NumNeighbors, List[int], Dict[EdgeType, List[int]]]
 
 class NeighborSampler(BaseSampler):
     r"""An implementation of an in-memory (heterogeneous) neighbor sampler used
-    by :class:`~torch_geometric.loader.NeighborLoader`."""
+    by :class:`~torch_geometric.loader.NeighborLoader`.
+    """
     def __init__(
         self,
         data: Union[Data, HeteroData, Tuple[FeatureStore, GraphStore]],
@@ -68,13 +69,26 @@ class NeighborSampler(BaseSampler):
             self.num_nodes = data.num_nodes
 
             self.node_time: Optional[Tensor] = None
+            self.edge_time: Optional[Tensor] = None
+
             if time_attr is not None:
-                self.node_time = data[time_attr]
+                if data.is_node_attr(time_attr):
+                    self.node_time = data[time_attr]
+                elif data.is_edge_attr(time_attr):
+                    self.edge_time = data[time_attr]
+                else:
+                    raise ValueError(
+                        f"The time attribute '{time_attr}' is neither a "
+                        f"node-level or edge-level attribute")
 
             # Convert the graph data into CSC format for sampling:
             self.colptr, self.row, self.perm = to_csc(
                 data, device='cpu', share_memory=share_memory,
-                is_sorted=is_sorted, src_node_time=self.node_time)
+                is_sorted=is_sorted, src_node_time=self.node_time,
+                edge_time=self.edge_time)
+
+            if self.edge_time is not None and self.perm is not None:
+                self.edge_time = self.edge_time[self.perm]
 
             self.edge_weight: Optional[Tensor] = None
             if weight_attr is not None:
@@ -88,8 +102,32 @@ class NeighborSampler(BaseSampler):
             self.num_nodes = {k: data[k].num_nodes for k in self.node_types}
 
             self.node_time: Optional[Dict[NodeType, Tensor]] = None
+            self.edge_time: Optional[Dict[EdgeType, Tensor]] = None
+
             if time_attr is not None:
-                self.node_time = data.collect(time_attr)
+                is_node_level_time = is_edge_level_time = False
+
+                for store in data.node_stores:
+                    if time_attr in store:
+                        is_node_level_time = True
+                for store in data.edge_stores:
+                    if time_attr in store:
+                        is_edge_level_time = True
+
+                if is_node_level_time and is_edge_level_time:
+                    raise ValueError(
+                        f"The time attribute '{time_attr}' holds both "
+                        f"node-level and edge-level information")
+
+                if not is_node_level_time and not is_edge_level_time:
+                    raise ValueError(
+                        f"The time attribute '{time_attr}' is neither a "
+                        f"node-level or edge-level attribute")
+
+                if is_node_level_time:
+                    self.node_time = data.collect(time_attr)
+                else:
+                    self.edge_time = data.collect(time_attr)
 
             # Conversion to/from C++ string type: Since C++ cannot take
             # dictionaries with tuples as key as input, edge type triplets need
@@ -100,10 +138,19 @@ class NeighborSampler(BaseSampler):
             # Convert the graph data into CSC format for sampling:
             colptr_dict, row_dict, self.perm = to_hetero_csc(
                 data, device='cpu', share_memory=share_memory,
-                is_sorted=is_sorted, node_time_dict=self.node_time)
+                is_sorted=is_sorted, node_time_dict=self.node_time,
+                edge_time_dict=self.edge_time)
+
             self.row_dict = remap_keys(row_dict, self.to_rel_type)
             self.colptr_dict = remap_keys(colptr_dict, self.to_rel_type)
 
+            if self.edge_time is not None:
+                for edge_type, edge_time in self.edge_time.items():
+                    if self.perm.get(edge_type, None) is not None:
+                        edge_time = edge_time[self.perm[edge_type]]
+                        self.edge_time[edge_type] = edge_time
+                self.edge_time = remap_keys(self.edge_time, self.to_rel_type)
+
             self.edge_weight: Optional[Dict[EdgeType, Tensor]] = None
             if weight_attr is not None:
                 self.edge_weight = data.collect(weight_attr)
@@ -118,11 +165,10 @@ class NeighborSampler(BaseSampler):
             feature_store, graph_store = data
 
             # Obtain graph metadata:
-            node_attrs = feature_store.get_all_tensor_attrs()
-            self.node_types = list(set(attr.group_name for attr in node_attrs))
+            attrs = [attr for attr in feature_store.get_all_tensor_attrs()]
 
             edge_attrs = graph_store.get_all_edge_attrs()
-            self.edge_types = list(set(attr.edge_type for attr in edge_attrs))
+            self.edge_types = list({attr.edge_type for attr in edge_attrs})
 
             if weight_attr is not None:
                 raise NotImplementedError(
@@ -148,26 +194,39 @@ class NeighborSampler(BaseSampler):
 
                 # We obtain all features with `node_attr.name=time_attr`:
                 time_attrs = [
-                    copy.copy(attr) for attr in node_attrs
+                    copy.copy(attr) for attr in attrs
                     if attr.attr_name == time_attr
                 ]
 
             if not self.is_hetero:
+                self.node_types = [None]
                 self.num_nodes = max(edge_attrs[0].size)
                 self.edge_weight: Optional[Tensor] = None
 
                 self.node_time: Optional[Tensor] = None
-                if time_attr is not None and len(time_attrs) != 1:
-                    raise ValueError("Temporal sampling specified but did "
-                                     "not find any temporal data")
+                self.edge_time: Optional[Tensor] = None
 
+                if time_attr is not None:
+                    if len(time_attrs) != 1:
+                        raise ValueError("Temporal sampling specified but did "
+                                         "not find any temporal data")
                     time_attrs[0].index = None  # Reset index for full data.
                     time_tensor = feature_store.get_tensor(time_attrs[0])
-                    self.node_time = time_tensor
+                    # Currently, we determine whether to use node-level or
+                    # edge-level temporal sampling based on the attribute name.
+                    if time_attr == 'time':
+                        self.node_time = time_tensor
+                    else:
+                        self.edge_time = time_tensor
 
                 self.row, self.colptr, self.perm = graph_store.csc()
 
             else:
+                node_types = [
+                    attr.group_name for attr in attrs
+                    if isinstance(attr.group_name, str)
+                ]
+                self.node_types = list(set(node_types))
                 self.num_nodes = {
                     node_type: remote_backend_utils.size(*data, node_type)
                     for node_type in self.node_types
@@ -175,16 +234,28 @@ class NeighborSampler(BaseSampler):
                 self.edge_weight: Optional[Dict[EdgeType, Tensor]] = None
 
                 self.node_time: Optional[Dict[NodeType, Tensor]] = None
+                self.edge_time: Optional[Dict[EdgeType, Tensor]] = None
+
                 if time_attr is not None:
                     for attr in time_attrs:  # Reset index for full data.
                         attr.index = None
+
                     time_tensors = feature_store.multi_get_tensor(time_attrs)
-                    self.node_time = {
-                        time_attr.group_name: time_tensor
-                        for time_attr, time_tensor in zip(
-                            time_attrs, time_tensors)
+                    time = {
+                        attr.group_name: time_tensor
+                        for attr, time_tensor in zip(time_attrs, time_tensors)
                     }
 
+                    group_names = [attr.group_name for attr in time_attrs]
+                    if all([isinstance(g, str) for g in group_names]):
+                        self.node_time = time
+                    elif all([isinstance(g, tuple) for g in group_names]):
+                        self.edge_time = time
+                    else:
+                        raise ValueError(
+                            f"Found time attribute '{time_attr}' for both "
+                            f"node-level and edge-level types")
+
                 # Conversion to/from C++ string type (see above):
                 self.to_rel_type = {k: '__'.join(k) for k in self.edge_types}
                 self.to_edge_type = {v: k for k, v in self.to_rel_type.items()}
@@ -193,6 +264,11 @@ class NeighborSampler(BaseSampler):
                 self.row_dict = remap_keys(row_dict, self.to_rel_type)
                 self.colptr_dict = remap_keys(colptr_dict, self.to_rel_type)
 
+        if (self.edge_time is not None
+                and not torch_geometric.typing.WITH_EDGE_TIME_NEIGHBOR_SAMPLE):
+            raise ImportError("Edge-level temporal sampling requires a "
+                              "more recent 'pyg-lib' installation")
+
         if (self.edge_weight is not None
                 and not torch_geometric.typing.WITH_WEIGHTED_NEIGHBOR_SAMPLE):
             raise ImportError("Weighted neighbor sampling requires "
@@ -223,11 +299,11 @@ class NeighborSampler(BaseSampler):
             return True
 
         # self.data_type == DataType.remote
-        return self.node_types != [None]
+        return self.edge_types != [None]
 
     @property
     def is_temporal(self) -> bool:
-        return self.node_time is not None
+        return self.node_time is not None or self.edge_time is not None
 
     @property
     def disjoint(self) -> bool:
@@ -276,7 +352,8 @@ class NeighborSampler(BaseSampler):
         **kwargs,
     ) -> Union[SamplerOutput, HeteroSamplerOutput]:
         r"""Implements neighbor sampling by calling either :obj:`pyg-lib` (if
-        installed) or :obj:`torch-sparse` (if installed) sampling routines."""
+        installed) or :obj:`torch-sparse` (if installed) sampling routines.
+        """
         if isinstance(seed, dict):  # Heterogeneous sampling:
             # TODO Support induced subgraph sampling in `pyg-lib`.
             if (torch_geometric.typing.WITH_PYG_LIB
@@ -294,8 +371,10 @@ class NeighborSampler(BaseSampler):
                     seed,
                     self.num_neighbors.get_mapped_values(self.edge_types),
                     self.node_time,
-                    seed_time,
                 )
+                if torch_geometric.typing.WITH_EDGE_TIME_NEIGHBOR_SAMPLE:
+                    args += (self.edge_time, )
+                args += (seed_time, )
                 if torch_geometric.typing.WITH_WEIGHTED_NEIGHBOR_SAMPLE:
                     args += (self.edge_weight, )
                 args += (
@@ -323,10 +402,16 @@ class NeighborSampler(BaseSampler):
 
             elif torch_geometric.typing.WITH_TORCH_SPARSE:
                 if self.disjoint:
-                    raise ValueError("'disjoint' sampling not supported for "
-                                     "neighbor sampling via 'torch-sparse'. "
-                                     "Please install 'pyg-lib' for improved "
-                                     "and optimized sampling routines.")
+                    if self.subgraph_type == SubgraphType.induced:
+                        raise ValueError("'disjoint' sampling not supported "
+                                         "for neighbor sampling with "
+                                         "`subgraph_type='induced'`")
+                    else:
+                        raise ValueError("'disjoint' sampling not supported "
+                                         "for neighbor sampling via "
+                                         "'torch-sparse'. Please install "
+                                         "'pyg-lib' for improved and "
+                                         "optimized sampling routines.")
 
                 out = torch.ops.torch_sparse.hetero_neighbor_sample(
                     self.node_types,
@@ -374,8 +459,10 @@ class NeighborSampler(BaseSampler):
                     seed.to(self.colptr.dtype),
                     self.num_neighbors.get_mapped_values(),
                     self.node_time,
-                    seed_time,
                 )
+                if torch_geometric.typing.WITH_EDGE_TIME_NEIGHBOR_SAMPLE:
+                    args += (self.edge_time, )
+                args += (seed_time, )
                 if torch_geometric.typing.WITH_WEIGHTED_NEIGHBOR_SAMPLE:
                     args += (self.edge_weight, )
                 args += (
@@ -431,56 +518,6 @@ class NeighborSampler(BaseSampler):
                 num_sampled_edges=num_sampled_edges,
             )
 
-    def _sample_one_hop(
-        self,
-        input_nodes: Tensor,
-        num_neighbors: int,
-        seed_time: Optional[Tensor] = None,
-        edge_type: Optional[EdgeType] = None,
-    ) -> SamplerOutput:
-        r"""Implements one-hop neighbor sampling for a set of input nodes for a
-        specific edge type.
-        """
-        rel_type = '__'.join(edge_type) if self.is_hetero else None
-
-        if not self.is_hetero:
-            colptr = self.colptr
-            row = self.row
-            node_time = self.node_time
-        else:
-            rel_type = '__'.join(edge_type)
-            colptr = self.colptr_dict[rel_type]
-            row = self.row_dict[rel_type]
-            node_time = self.node_time.get(edge_type[2], None)
-
-        out = torch.ops.pyg.dist_neighbor_sample(
-            colptr,
-            row,
-            input_nodes.to(colptr.dtype),
-            num_neighbors,
-            node_time,
-            seed_time,
-            None,  # TODO: edge_weight
-            True,  # csc
-            self.replace,
-            self.subgraph_type != SubgraphType.induced,
-            self.disjoint and node_time is not None,
-            self.temporal_strategy,
-        )
-        node, edge, cumsum_neighbors_per_node = out
-
-        if self.disjoint:
-            batch, node = node.t().contiguous()
-
-        return SamplerOutput(
-            node=node,
-            row=None,
-            col=None,
-            edge=edge,
-            batch=batch,
-            metadata=(cumsum_neighbors_per_node, ),
-        )
-
 
 # Sampling Utilities ##########################################################
 
@@ -491,7 +528,8 @@ def node_sample(
 ) -> Union[SamplerOutput, HeteroSamplerOutput]:
     r"""Performs sampling from a :class:`NodeSamplerInput`, leveraging a
     sampling function that accepts a seed and (optionally) a seed time as
-    input. Returns the output of this sampling procedure."""
+    input. Returns the output of this sampling procedure.
+    """
     if inputs.input_type is not None:  # Heterogeneous sampling:
         seed = {inputs.input_type: inputs.node}
         seed_time = None
@@ -516,7 +554,8 @@ def edge_sample(
     neg_sampling: Optional[NegativeSampling] = None,
 ) -> Union[SamplerOutput, HeteroSamplerOutput]:
     r"""Performs sampling from an edge sampler input, leveraging a sampling
-    function of the same signature as `node_sample`."""
+    function of the same signature as `node_sample`.
+    """
     input_id = inputs.input_id
     src = inputs.row
     dst = inputs.col
@@ -554,7 +593,7 @@ def edge_sample(
                 src_node_time = node_time
 
             src_neg = neg_sample(src, neg_sampling, num_src_nodes, src_time,
-                                 src_node_time)
+                                 src_node_time, endpoint='src')
             src = torch.cat([src, src_neg], dim=0)
 
             if isinstance(node_time, dict):
@@ -563,7 +602,7 @@ def edge_sample(
                 dst_node_time = node_time
 
             dst_neg = neg_sample(dst, neg_sampling, num_dst_nodes, dst_time,
-                                 dst_node_time)
+                                 dst_node_time, endpoint='dst')
             dst = torch.cat([dst, dst_neg], dim=0)
 
             if edge_label is None:
@@ -584,7 +623,7 @@ def edge_sample(
                 dst_node_time = node_time
 
             dst_neg = neg_sample(dst, neg_sampling, num_dst_nodes, dst_time,
-                                 dst_node_time)
+                                 dst_node_time, endpoint='dst')
             dst = torch.cat([dst, dst_neg], dim=0)
 
             assert edge_label is None
@@ -592,7 +631,7 @@ def edge_sample(
             if edge_label_time is not None:
                 dst_time = edge_label_time.repeat(1 + neg_sampling.amount)
 
-    # Heterogeneus Neighborhood Sampling ######################################
+    # Heterogeneous Neighborhood Sampling #####################################
 
     if input_type is not None:
         seed_time_dict = None
@@ -685,7 +724,7 @@ def edge_sample(
                 src_time,
             )
 
-    # Homogeneus Neighborhood Sampling ########################################
+    # Homogeneous Neighborhood Sampling #######################################
 
     else:
 
@@ -742,12 +781,13 @@ def neg_sample(
     num_nodes: int,
     seed_time: Optional[Tensor],
     node_time: Optional[Tensor],
+    endpoint: Literal['str', 'dst'],
 ) -> Tensor:
     num_neg = math.ceil(seed.numel() * neg_sampling.amount)
 
     # TODO: Do not sample false negatives.
     if node_time is None:
-        return neg_sampling.sample(num_neg, num_nodes)
+        return neg_sampling.sample(num_neg, endpoint, num_nodes)
 
     # If we are in a temporal-sampling scenario, we need to respect the
     # timestamp of the given nodes we can use as negative examples.
@@ -761,7 +801,7 @@ def neg_sample(
     num_samples = math.ceil(neg_sampling.amount)
     seed_time = seed_time.view(1, -1).expand(num_samples, -1)
 
-    out = neg_sampling.sample(num_samples * seed.numel(), num_nodes)
+    out = neg_sampling.sample(num_samples * seed.numel(), endpoint, num_nodes)
     out = out.view(num_samples, seed.numel())
     mask = node_time[out] > seed_time  # holds all invalid samples.
     neg_sampling_complete = False
@@ -772,7 +812,7 @@ def neg_sample(
             break
 
         # Greedily search for alternative negatives.
-        out[mask] = tmp = neg_sampling.sample(num_invalid, num_nodes)
+        out[mask] = tmp = neg_sampling.sample(num_invalid, endpoint, num_nodes)
         mask[mask.clone()] = node_time[tmp] >= seed_time[mask]
 
     if not neg_sampling_complete:  # pragma: no cover
