@@ -2,7 +2,6 @@ from typing import Any, Dict, Tuple
 
 import numpy as np
 import torch
-from scipy.linalg import expm
 from torch import Tensor
 
 from torch_geometric.data import Data
@@ -22,7 +21,7 @@ from torch_geometric.utils import (
 @functional_transform('gdc')
 class GDC(BaseTransform):
     r"""Processes the graph via Graph Diffusion Convolution (GDC) from the
-    `"Diffusion Improves Graph Learning" <https://www.kdd.in.tum.de/gdc>`_
+    `"Diffusion Improves Graph Learning" <https://arxiv.org/abs/1911.05485>`_
     paper (functional name: :obj:`gdc`).
 
     .. note::
@@ -80,10 +79,12 @@ class GDC(BaseTransform):
         normalization_in: str = 'sym',
         normalization_out: str = 'col',
         diffusion_kwargs: Dict[str, Any] = dict(method='ppr', alpha=0.15),
-        sparsification_kwargs: Dict[str, Any] = dict(method='threshold',
-                                                     avg_degree=64),
+        sparsification_kwargs: Dict[str, Any] = dict(
+            method='threshold',
+            avg_degree=64,
+        ),
         exact: bool = True,
-    ):
+    ) -> None:
         self.self_loop_weight = self_loop_weight
         self.normalization_in = normalization_in
         self.normalization_out = normalization_out
@@ -96,8 +97,11 @@ class GDC(BaseTransform):
 
     @torch.no_grad()
     def forward(self, data: Data) -> Data:
-        N = data.num_nodes
+        assert data.edge_index is not None
         edge_index = data.edge_index
+        N = data.num_nodes
+        assert N is not None
+
         if data.edge_attr is None:
             edge_weight = torch.ones(edge_index.size(1),
                                      device=edge_index.device)
@@ -189,13 +193,13 @@ class GDC(BaseTransform):
 
         return edge_index, edge_weight
 
-    def diffusion_matrix_exact(
+    def diffusion_matrix_exact(  # noqa: D417
         self,
         edge_index: Tensor,
         edge_weight: Tensor,
         num_nodes: int,
         method: str,
-        **kwargs,
+        **kwargs: Any,
     ) -> Tensor:
         r"""Calculate the (dense) diffusion on a given sparse graph.
         Note that these exact variants are not scalable. They densify the
@@ -262,14 +266,14 @@ class GDC(BaseTransform):
 
         return diff_matrix
 
-    def diffusion_matrix_approx(
+    def diffusion_matrix_approx(  # noqa: D417
         self,
         edge_index: Tensor,
         edge_weight: Tensor,
         num_nodes: int,
         normalization: str,
         method: str,
-        **kwargs,
+        **kwargs: Any,
     ) -> Tuple[Tensor, Tensor]:
         r"""Calculate the approximate, sparse diffusion on a given sparse
         graph.
@@ -333,20 +337,20 @@ class GDC(BaseTransform):
 
         elif method == 'heat':
             raise NotImplementedError(
-                ('Currently no fast heat kernel is implemented. You are '
-                 'welcome to create one yourself, e.g., based on '
-                 '"Kloster and Gleich: Heat kernel based community detection '
-                 '(KDD 2014)."'))
+                'Currently no fast heat kernel is implemented. You are '
+                'welcome to create one yourself, e.g., based on '
+                '"Kloster and Gleich: Heat kernel based community detection '
+                '(KDD 2014)."')
         else:
             raise ValueError(f"Approximate GDC diffusion '{method}' unknown")
 
         return edge_index, edge_weight
 
-    def sparsify_dense(
+    def sparsify_dense(  # noqa: D417
         self,
         matrix: Tensor,
         method: str,
-        **kwargs,
+        **kwargs: Any,
     ) -> Tuple[Tensor, Tensor]:
         r"""Sparsifies the given dense matrix.
 
@@ -411,19 +415,19 @@ class GDC(BaseTransform):
 
         return edge_index, edge_weight
 
-    def sparsify_sparse(
+    def sparsify_sparse(  # noqa: D417
         self,
         edge_index: Tensor,
         edge_weight: Tensor,
         num_nodes: int,
         method: str,
-        **kwargs,
+        **kwargs: Any,
     ) -> Tuple[Tensor, Tensor]:
         r"""Sparsifies a given sparse graph further.
 
         Args:
-            edge_index (LongTensor): The edge indices.
-            edge_weight (Tensor): One-dimensional edge weights.
+            edge_index (torch.Tensor): The edge indices.
+            edge_weight (torch.Tensor): One-dimensional edge weights.
             num_nodes (int): Number of nodes.
             method (str): Method of sparsification:
 
@@ -441,8 +445,11 @@ class GDC(BaseTransform):
         """
         if method == 'threshold':
             if 'eps' not in kwargs.keys():
-                kwargs['eps'] = self.__calculate_eps__(edge_weight, num_nodes,
-                                                       kwargs['avg_degree'])
+                kwargs['eps'] = self.__calculate_eps__(
+                    edge_weight,
+                    num_nodes,
+                    kwargs['avg_degree'],
+                )
 
             remaining_edge_idx = (edge_weight >= kwargs['eps']).nonzero(
                 as_tuple=False).flatten()
@@ -465,6 +472,8 @@ class GDC(BaseTransform):
 
         :rtype: (:class:`Tensor`)
         """
+        from scipy.linalg import expm
+
         if symmetric:
             e, V = torch.linalg.eigh(matrix, UPLO='U')
             diff_mat = V @ torch.diag(e.exp()) @ V.t()
@@ -494,4 +503,4 @@ class GDC(BaseTransform):
 
         left = sorted_edges[avg_degree * num_nodes - 1]
         right = sorted_edges[avg_degree * num_nodes]
-        return (left + right) / 2.0
+        return float(left + right) / 2.0
